@@ -210,6 +210,11 @@ static void print_version(void)
     g_print(_("\tUsing automatic archiving.\n"));
 #else
     g_print(_("\tNot using archiving.\n"));
+#endif
+#ifdef HAVE_LIBICAL
+    g_print(_("\tUsing operating system package libical.\n"));
+#else
+    g_print(_("\tUsing Orage local version of libical.\n"));
 #endif
     g_print("\n");
 }
@@ -475,14 +480,14 @@ int main(int argc, char *argv[])
         gtk_widget_realize(((CalWin *)g_par.xfcal)->mWindow);
         gtk_widget_hide(((CalWin *)g_par.xfcal)->mWindow);
     }
-    reset_orage_day_change(FALSE); /* first day change after we start */
     alarm_read();
+    orage_day_change(FALSE); /* first day change after we start */
     mCalendar_month_changed_cb(
             (GtkCalendar *)((CalWin *)g_par.xfcal)->mCalendar, NULL);
 
     /* start monitoring foreign file updates if we have foreign files */
     if (g_par.foreign_count)
-        g_timeout_add(30*1000, (GtkFunction)orage_foreign_files_check, NULL);
+        g_timeout_add_seconds(30, (GtkFunction)orage_foreign_files_check, NULL);
                                                         
     /* let's check if I got filename as a parameter */
     initialized = TRUE;
